@@ -21,6 +21,46 @@ def test_prefixes(runner):
     assert re.search(r'--bar\s+das bar', result.output) is not None
 
 
+def test_invalid_option(runner):
+    try:
+        @click.command()
+        @click.option('foo')
+        def cli(foo):
+            pass
+    except TypeError as e:
+        assert 'No options defined but a name was passed (foo).' \
+            in str(e)
+    else:
+        assert False, 'Expected a type error because of an invalid option.'
+
+
+def test_invalid_nargs(runner):
+    try:
+        @click.command()
+        @click.option('--foo', nargs=-1)
+        def cli(foo):
+            pass
+    except TypeError as e:
+        assert 'Options cannot have nargs < 0' in str(e)
+    else:
+        assert False, 'Expected a type error because of an invalid option.'
+
+
+def test_nargs_tup_composite_mult(runner):
+    @click.command()
+    @click.option('--item', type=(str, int), multiple=True)
+    def copy(item):
+        for item in item:
+            click.echo('name=%s id=%d' % item)
+
+    result = runner.invoke(copy, ['--item', 'peter', '1', '--item', 'max', '2'])
+    assert not result.exception
+    assert result.output.splitlines() == [
+        'name=peter id=1',
+        'name=max id=2',
+    ]
+
+
 def test_counting(runner):
     @click.command()
     @click.option('-v', count=True, help='Verbosity',
@@ -102,6 +142,21 @@ def test_multiple_envvar(runner):
     assert result.output == 'foo|bar\n'
 
 
+def test_multiple_default_help(runner):
+    @click.command()
+    @click.option("--arg1", multiple=True, default=('foo', 'bar'),
+                  show_default=True)
+    @click.option("--arg2", multiple=True, default=(1, 2), type=int,
+                  show_default=True)
+    def cmd(arg, arg2):
+        pass
+
+    result = runner.invoke(cmd, ['--help'])
+    assert not result.exception
+    assert "foo, bar" in result.output
+    assert "1, 2" in result.output
+
+
 def test_nargs_envvar(runner):
     @click.command()
     @click.option('--arg', nargs=2)
@@ -204,3 +259,35 @@ def test_multiline_help(runner):
     assert '  --foo TEXT  hello' in out
     assert '              i am' in out
     assert '              multiline' in out
+
+
+def test_argument_custom_class(runner):
+    class CustomArgument(click.Argument):
+        def get_default(self, ctx):
+            '''a dumb override of a default value for testing'''
+            return 'I am a default'
+
+    @click.command()
+    @click.argument('testarg', cls=CustomArgument, default='you wont see me')
+    def cmd(testarg):
+        click.echo(testarg)
+
+    result = runner.invoke(cmd)
+    assert 'I am a default' in result.output
+    assert 'you wont see me' not in result.output
+
+
+def test_option_custom_class(runner):
+    class CustomOption(click.Option):
+        def get_help_record(self, ctx):
+            '''a dumb override of a help text for testing'''
+            return ('--help', 'I am a help text')
+
+    @click.command()
+    @click.option('--testoption', cls=CustomOption, help='you wont see me')
+    def cmd(testoption):
+        click.echo(testoption)
+
+    result = runner.invoke(cmd, ['--help'])
+    assert 'I am a help text' in result.output
+    assert 'you wont see me' not in result.output
